/**
 * Next function supports optional `ctx` replacement for following middleware.
 */
export declare type Next<T> = () => Promise<T>;
/**
 * Middleware function pattern.
 */
export declare type Middleware<T, U> = (ctx: T, next: Next<U>) => U | Promise<U>;
/**
 * Final function has no `next()`.
 */
export declare type Done<T, U> = (ctx: T) => U | Promise<U>;
/**
 * Composed function signature.
 */
export declare type Composed<T, U> = (ctx: T, done: Done<T, U>) => Promise<U>;
/**
 * Compose an array of middleware functions into a single function.
 */
export declare function compose<T, U>(middleware: Array<Middleware<T, U>>, debug?: boolean): Composed<T, U>;
